@@ -8,7 +8,7 @@
 #' @examples
 #' #Fake example
 fliplr = function(x) {
-  if(class(x) == "matrix") {
+  if(length(dim(x)) == 2) {
     x[,ncol(x):1]
   } else {
     x[,ncol(x):1,]
@@ -26,7 +26,7 @@ fliplr = function(x) {
 #' @examples
 #' #Fake example
 flipud = function(x) {
-  if(class(x) == "matrix") {
+  if(length(dim(x)) == 2) {
     x[nrow(x):1,]
   } else {
     x[nrow(x):1,,]
