@@ -54,58 +54,30 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
   position_list$xvec = scene$x 
   position_list$yvec = scene$y
   position_list$zvec = scene$z
-  rvec = scene$radius
+  
   shapevec = unlist(lapply(tolower(scene$shape),switch,
-                           "sphere" = 1,"xy_rect" = 2, "xz_rect" = 3,"yz_rect" = 4,"box" = 5, "triangle" = 6, 
-                           "obj" = 7, "objcolor" = 8, "disk" = 9, "cylinder" = 10, "ellipsoid" = 11,
-                           "objvertexcolor" = 12, "cone" = 13, "curve" = 14, "csg_object" = 15, "ply" = 16,
-                           "mesh3d" = 17, "raymesh" = 18))
-  typevec = unlist(lapply(tolower(scene$type),switch,
-                          "diffuse" = 1,"metal" = 2,"dielectric" = 3, 
-                          "oren-nayar" = 4, "light" = 5, "microfacet" = 6, 
-                          "glossy" = 7, "spotlight" = 8, "hair" = 9, "microfacet_transmission" = 10))
-  sigmavec = unlist(scene$sigma)
+                           "sphere" = 1,"xy_rect" = 2, "xz_rect" = 3,"yz_rect" = 4,"box" = 5, 
+                           "obj" = 6, "disk" = 7, "cylinder" = 8, "ellipsoid" = 9,
+                          "curve" = 10, "csg_object" = 11, "ply" = 12,
+                           "mesh3d" = 13, "raymesh" = 14, stop(sprintf("Shape not recognized"))))
+  scene$shape = shapevec
+  
+  # Convert shapes to enums
+  typevec = rep(0,nrow(scene))
+  
+  for(i in seq_len(nrow(scene))) {
+    typevec[i] = switch(scene$material[[i]]$type,
+                        "diffuse" = 1L,"metal" = 2L,"dielectric" = 3L,
+                        "oren-nayar" = 4L, "light" = 5L, "mf" = 6L,
+                        "glossy" = 7L, "spotlight" = 8L, "hair" = 9L, "mf-t" = 10L,
+                        stop(sprintf("Material type `%s` not found",scene$material[[i]]$type)))
+    scene$material[[i]]$type = typevec[i]
+  }
   
   if(!tonemap %in% c("gamma","reinhold","uncharted", "hbd", "raw")) {
     stop("tonemap value ", tonemap, " not recognized")
   }
   toneval = switch(tonemap, "gamma" = 1,"reinhold" = 2,"uncharted" = 3,"hbd" = 4, "raw" = 5)
-  proplist = scene$properties
-  
-  checkeredlist = scene$checkercolor
-  checkeredbool = purrr::map_lgl(checkeredlist,.f = ~all(!is.na(.x)))
-  
-  #glossy 
-  glossyinfo = scene$glossyinfo
-  
-  #gradient handler
-  gradient_info = list()
-  gradient_info$gradient_colors = scene$gradient_color
-  gradient_info$isgradient = purrr::map_lgl(gradient_info$gradient_colors,.f = ~all(!is.na(.x)))
-  gradient_info$gradient_trans = scene$gradient_transpose
-  gradient_info$is_world_gradient = scene$world_gradient
-  gradient_info$gradient_control_points = scene$gradient_point_info
-  gradient_info$type = unlist(lapply(tolower(scene$gradient_type),switch,
-                                     "hsv" = TRUE, "rgb" = FALSE, FALSE))
-  #noise handler
-  noisebool = purrr::map_lgl(scene$noise, .f = ~.x > 0)
-  noisevec = scene$noise
-  noisephasevec = scene$noisephase * pi/180
-  noiseintvec = scene$noiseintensity
-  noisecolorlist = scene$noisecolor
-  
-  #rotation handler
-  rot_angle_list = scene$angle
-  
-  #fog handler
-  fog_bool = scene$fog
-  fog_vec = scene$fogdensity
-  
-  #flip handler
-  flip_vec = scene$flipped
-  
-  #light handler
-  light_prop_vec =  scene$lightintensity
   
   if(!any(typevec == 5) && !any(typevec == 8) && is.null(ambient_light) && is.null(environment_light)) {
     ambient_light = TRUE
@@ -116,11 +88,14 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
   }
   
   #alpha texture handler
-  alpha_array_list = scene$alphaimage
-  alpha_tex_bool = purrr::map_lgl(alpha_array_list,.f = ~is.array(.x[[1]]))
-  alpha_filename_bool = purrr::map_lgl(alpha_array_list,.f = ~is.character(.x[[1]]))
-  alpha_temp_file_names = purrr::map_chr(alpha_tex_bool, .f = (function(.x) tempfile(fileext = ".png")))
-  for(i in 1:length(alpha_array_list)) {
+  alpha_array_list = vector(mode="list", length(nrow(scene)))
+  alpha_tex_bool = vector(mode = "logical", length(nrow(scene)))
+  alpha_filename_bool = vector(mode = "logical", length(nrow(scene)))
+  alpha_temp_file_names = tempfile(sprintf("alphatemp%i",seq_len(nrow(scene))),fileext = ".png")
+  for(i in seq_len(nrow(scene))) {
+    alpha_array_list[i] = scene$material[[i]]$alphaimage
+    alpha_tex_bool[i] = is.array(alpha_array_list[[i]])
+    alpha_filename_bool[i] = is.character(alpha_array_list[[i]]) && !is.na(alpha_array_list[[i]])
     if(alpha_tex_bool[i]) {
       if(length(dim(alpha_array_list[[i]][[1]])) == 2) {
         png::writePNG(fliplr(t(alpha_array_list[[i]][[1]])), alpha_temp_file_names[i])
@@ -134,8 +109,8 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
       } else {
         stop("alpha texture dims: c(", paste(dim(alpha_array_list[[i]][[1]]),collapse=", "), ") not valid for texture.")
       }
-    }
-    if(alpha_filename_bool[i]) {
+      scene$material[[i]]$alphaimage = TRUE
+    } else if(alpha_filename_bool[i]) {
       if(any(!file.exists(path.expand(alpha_array_list[[i]][[1]])) & nchar(alpha_array_list[[i]][[1]]) > 0)) {
         stop(paste0("Cannot find the following texture file:\n",
                     paste(alpha_array_list[[i]][[1]], collapse="\n")))
@@ -146,7 +121,10 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
         temp_array[,,2] = temp_array[,,4]
         temp_array[,,3] = temp_array[,,4]
       }
+      scene$material[[i]]$alphaimage = TRUE
       png::writePNG(temp_array,alpha_temp_file_names[i])
+    } else {
+      scene$material[[i]]$alphaimage = FALSE
     }
   }
   alpha_tex_bool = alpha_tex_bool | alpha_filename_bool
@@ -155,11 +133,14 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
   alphalist$alpha_tex_bool = alpha_tex_bool
   
   #texture handler
-  image_array_list = scene$image
-  image_tex_bool = purrr::map_lgl(image_array_list,.f = ~is.array(.x))
-  image_filename_bool = purrr::map_lgl(image_array_list,.f = ~is.character(.x))
-  temp_file_names = purrr::map_chr(image_tex_bool,.f = ~ifelse(.x, tempfile(fileext = ".png"),""))
-  for(i in 1:length(image_array_list)) {
+  image_array_list = vector(mode="list", length(nrow(scene))) #scene$image
+  image_tex_bool = vector(mode = "logical", length(nrow(scene)))
+  image_filename_bool = vector(mode = "logical", length(nrow(scene)))
+  temp_file_names = tempfile(sprintf("imagetemp%i",seq_len(nrow(scene))),fileext = ".png")
+  for(i in seq(nrow(scene))) {
+    image_array_list[i] = scene$material[[i]]$image
+    image_tex_bool[i] = is.array(image_array_list[[i]])
+    image_filename_bool[i] = is.character(image_array_list[[i]]) && !is.na(image_array_list[[i]])
     if(image_tex_bool[i]) {
       if(dim(image_array_list[[i]])[3] == 4) {
         png::writePNG(fliplr(aperm(image_array_list[[i]][,,1:3],c(2,1,3))),temp_file_names[i])
@@ -170,28 +151,35 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
           image_array_list[[i]][,,3] = image_array_list[[i]][,,4]
           png::writePNG(fliplr(aperm(image_array_list[[i]][,,1:3],c(2,1,3))), alpha_temp_file_names[i])
           alphalist$alpha_tex_bool[i] = TRUE
+          scene$material[[i]]$alphaimage = TRUE
         }
       } else if(dim(image_array_list[[i]])[3] == 3){
         png::writePNG(fliplr(aperm(image_array_list[[i]],c(2,1,3))),temp_file_names[i])
       }
-    }
-    if(image_filename_bool[i]) {
+      scene$material[[i]]$image = TRUE
+    } else if(image_filename_bool[i]) {
       if(any(!file.exists(path.expand(image_array_list[[i]])) & nchar(image_array_list[[i]]) > 0)) {
         stop(paste0("Cannot find the following texture file:\n",
                     paste(image_array_list[[i]], collapse="\n")))
       }
       temp_file_names[i] = path.expand(image_array_list[[i]])
+      scene$material[[i]]$image = TRUE
+    } else {
+      scene$material[[i]]$image = FALSE
     }
   }
   image_tex_bool = image_tex_bool | image_filename_bool
-  image_repeat = scene$image_repeat
+  # image_repeat = scene$image_repeat
   
   #bump texture handler
-  bump_array_list = scene$bump_texture
-  bump_tex_bool = purrr::map_lgl(bump_array_list,.f = ~is.array(.x[[1]]))
-  bump_filename_bool = purrr::map_lgl(bump_array_list,.f = ~is.character(.x[[1]]))
-  bump_temp_file_names = purrr::map_chr(bump_tex_bool,.f = ~ifelse(.x, tempfile(fileext = ".png"),""))
-  for(i in 1:length(bump_array_list)) {
+  bump_array_list = vector(mode="list", length(nrow(scene)))
+  bump_tex_bool = vector(mode = "logical", length(nrow(scene)))
+  bump_filename_bool = vector(mode = "logical", length(nrow(scene)))
+  bump_temp_file_names = tempfile(sprintf("bumptemp%i",seq_len(nrow(scene))),fileext = ".png")
+  for(i in seq(nrow(scene))) {
+    bump_array_list[i] = scene$material[[i]]$bump_texture
+    bump_tex_bool[i] = is.array(bump_array_list[[i]])
+    bump_filename_bool[i] = is.character(bump_array_list[[i]])  && !is.na(bump_array_list[[i]])
     if(bump_tex_bool[i]) {
       bump_dims = dim(bump_array_list[[i]][[1]])
       if(length(bump_dims) == 2) {
@@ -208,29 +196,32 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
       } else if(bump_dims[3] == 3){
         png::writePNG(fliplr(aperm(temp_array,c(2,1,3))),bump_temp_file_names[i])
       }
-    }
-    if(bump_filename_bool[i]) {
+      scene$material[[i]]$bump_texture = TRUE
+    } else if(bump_filename_bool[i]) {
       if(any(!file.exists(path.expand(bump_array_list[[i]][[1]])) & nchar(bump_array_list[[i]][[1]]) > 0)) {
         stop(paste0("Cannot find the following texture file:\n",
                     paste(bump_array_list[[i]][[1]], collapse="\n")))
       }
+      scene$material[[i]]$bump_texture = TRUE
       bump_temp_file_names[i] = path.expand(bump_array_list[[i]][[1]])
+    } else {
+      scene$material[[i]]$bump_texture = FALSE
     }
   }
   bump_tex_bool = bump_tex_bool | bump_filename_bool
-  bump_intensity = scene$bump_intensity
   alphalist$bump_temp_file_names = bump_temp_file_names
   alphalist$bump_tex_bool = bump_tex_bool
-  alphalist$bump_intensity = bump_intensity
-  
+
   #roughness texture handler
-  roughness_array_list = scene$roughness_texture
-  rough_tex_bool = purrr::map_lgl(roughness_array_list,.f = ~is.array(.x[[1]]))
-  rough_filename_bool = purrr::map_lgl(roughness_array_list,.f = ~is.character(.x[[1]]))
-  rough_temp_file_names = purrr::map_chr(rough_tex_bool, .f = (function(.x) tempfile(fileext = ".png")))
-  for(i in 1:length(roughness_array_list)) {
+  roughness_array_list = vector(mode="list", length(nrow(scene)))
+  rough_tex_bool = vector(mode = "logical", length(nrow(scene)))
+  rough_filename_bool = vector(mode = "logical", length(nrow(scene)))
+  rough_temp_file_names = tempfile(sprintf("roughtemp%i",seq_len(nrow(scene))),fileext = ".png")
+  for(i in seq(nrow(scene))) {
+    roughness_array_list[i] = scene$material[[i]]$roughness_texture
+    rough_tex_bool[i] = is.array(roughness_array_list[[i]])
+    rough_filename_bool[i] = is.character(roughness_array_list[[i]])  && !is.na(roughness_array_list[[i]])
     if(rough_tex_bool[i]) {
-      tempgloss = glossyinfo[[i]]
       if(length(dim(roughness_array_list[[i]][[1]])) == 2) {
         png::writePNG(fliplr(t(roughness_array_list[[i]][[1]])), rough_temp_file_names[i])
       } else if(dim(roughness_array_list[[i]][[1]])[3] == 3) {
@@ -238,13 +229,16 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
       } else {
         stop("alpha texture dims: c(", paste(dim(roughness_array_list[[i]][[1]]),collapse=", "), ") not valid for texture.")
       }
-    }
-    if(rough_filename_bool[i]) {
+      scene$material[[i]]$roughness_texture = TRUE
+    } else if(rough_filename_bool[i]) {
       if(any(!file.exists(path.expand(roughness_array_list[[i]][[1]])) & nchar(roughness_array_list[[i]][[1]]) > 0)) {
         stop(paste0("Cannot find the following texture file:\n",
                     paste(roughness_array_list[[i]][[1]], collapse="\n")))
       }
       rough_temp_file_names[i] = path.expand(roughness_array_list[[i]][[1]])
+      scene$material[[i]]$roughness_texture = TRUE
+    } else{
+      scene$material[[i]]$roughness_texture = FALSE
     }
   }
   rough_tex_bool = rough_tex_bool | rough_filename_bool
@@ -252,52 +246,17 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
   roughness_list$rough_temp_file_names = rough_temp_file_names
   roughness_list$rough_tex_bool = rough_tex_bool
   
-  
-  #implicit sampling handler
-  implicit_vec = scene$implicit_sample
-  
-  #order rotation handler
-  order_rotation_list = scene$order_rotation
-  
-  #group handler
-  group_bool = purrr::map_lgl(scene$group_transform,.f = ~all(!is.na(.x)))
-  group_transform = scene$group_transform
-  
-  
-  #animation handler
-  animation_bool = purrr::map_lgl(scene$start_transform_animation,.f = ~all(!is.na(.x))) & 
-    purrr::map_lgl(scene$end_transform_animation,.f = ~all(!is.na(.x)))
-  start_transform_animation = scene$start_transform_animation
-  end_transform_animation = scene$end_transform_animation
-  animation_start_time = scene$start_time
-  animation_end_time = scene$end_time
-  
-  #triangle normal handler
-  tri_normal_bools = purrr::map2_lgl(shapevec,proplist,.f = ~.x == 6 && all(!is.na(.y)))
-  tri_color_vert = scene$tricolorinfo
-  is_tri_color = purrr::map_lgl(tri_color_vert,.f = ~all(!is.na(.x)))
-  
-  #obj handler
-  fileinfovec = scene$fileinfo
-  fileinfovec[is.na(fileinfovec)] = ""
-  objfilenamevec = purrr::map_chr(fileinfovec, path.expand)
-  if(any(!file.exists(objfilenamevec) & nchar(objfilenamevec) > 0)) {
-    stop(paste0("Cannot find the following obj/ply files:\n",
-                paste(objfilenamevec[!file.exists(objfilenamevec) & nchar(objfilenamevec) > 0], 
-                      collapse="\n")
-    ))
+  for(i in seq(nrow(scene))) {
+    fileinfovec = scene$shape_info[[i]]$fileinfo
+    if(!is.na(fileinfovec)) {
+      if(any(!file.exists(scene$shape_info[[i]]$fileinfo) & nchar(scene$shape_info[[i]]$fileinfo) > 0)) {
+        stop(paste0("Cannot find the following obj/ply file:\n",
+                    fileinfovec, collapse="\n"))
+      }
+    } 
   }
-  base_dir = function(x) {
-    dirname_processed = dirname(x)
-    if(dirname_processed == ".") {
-      return("")
-    } else {
-      return(dirname_processed)
-    }
-  }
-  objbasedirvec = purrr::map_chr(objfilenamevec, base_dir)
-  
-  #bg image handler
+
+  #Background image handler
   if(!is.null(environment_light)) {
     hasbackground = TRUE
     backgroundstring = path.expand(environment_light)
@@ -314,7 +273,6 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
   }
   
   #scale handler
-  scale_factor = scene$scale_factor
   
   if(length(lookfrom) != 3) {
     stop("lookfrom must be length-3 numeric vector")
@@ -397,13 +355,6 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
   camera_info$keep_colors = keep_colors
   camera_info$iso = iso
   
-  animation_info = list()
-  animation_info$animation_bool            = animation_bool            
-  animation_info$start_transform_animation = start_transform_animation 
-  animation_info$end_transform_animation   = end_transform_animation   
-  animation_info$animation_start_time      = animation_start_time      
-  animation_info$animation_end_time        = animation_end_time        
-  
   if(max_depth <= 0) {
     stop("max_depth must be greater than zero")
   }
@@ -416,20 +367,26 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
     if(any(shapevec[typevec == 8] > 4)) {
       stop("spotlights are only supported for spheres and rects")
     }
-    for(i in 1:length(proplist)) {
+    for(i in seq_len(nrow(scene))) {
       if(typevec[i] == 8) {
-        proplist[[i]][4:6] = proplist[[i]][4:6] - c(position_list$xvec[i],position_list$yvec[i],position_list$zvec[i]) 
+        scene$material[[i]]$properties[[1]][4:6] = scene$material[[i]]$properties[[1]][4:6] - 
+          c(position_list$xvec[i],position_list$yvec[i],position_list$zvec[i]) 
       }
     }
   }
   
   
   #Material ID handler; these must show up in increasing order.  Note, this will
-  #cause problems if `match` is every changed to return doubles when matching in
+  #cause problems if `match` is ever changed to return doubles when matching in
   #long vectors as has happened with `which` recently.
-  material_id = scene$material_id
-  material_id = as.integer(match(material_id, unique(material_id)) - 1L)
-  material_id_bool = !is.na(scene$material_id)
+  material_id = unlist(lapply(scene$shape_info, \(x) x$material_id))
+  is_na_mat = is.na(material_id)
+  material_id_increasing = as.integer(match(material_id, unique(material_id)) - 1L)
+  for(i in seq_len(nrow(scene))) {
+    if(!is_na_mat[i]) {
+      scene$shape_info[[i]]$material_id = material_id_increasing[i]
+    } 
+  }
   
   if(min_adaptive_size < 1) {
     warning("min_adaptive_size cannot be less than one: setting to one")
@@ -440,75 +397,33 @@ prepare_scene_list = function(scene, width = 400, height = 400, fov = 20,
     stop("min_variance cannot be less than zero")
   }
   
-  #CSG handler
-  csg_list = scene$csg_object
-  csg_info = list()
-  csg_info$csg = csg_list
-  
-  #mesh3d/raymesh handler
-  mesh_list = scene$mesh_info
-  
   
   scene_info = list()
   scene_info$ambient_light = ambient_light
-  scene_info$type = typevec
   scene_info$shape = shapevec
-  scene_info$radius = rvec
   scene_info$position_list = position_list
-  scene_info$properties = proplist
-  scene_info$n = length(typevec)
   scene_info$bghigh = backgroundhigh
   scene_info$bglow = backgroundlow
-  scene_info$ischeckered = checkeredbool
-  scene_info$checkercolors = checkeredlist
-  scene_info$gradient_info = gradient_info
-  scene_info$noise=noisevec
-  scene_info$isnoise=noisebool
-  scene_info$noisephase=noisephasevec
-  scene_info$noiseintensity=noiseintvec
-  scene_info$noisecolorlist = noisecolorlist
-  scene_info$angle = rot_angle_list
   scene_info$isimage = image_tex_bool
   scene_info$filelocation = temp_file_names
   scene_info$alphalist = alphalist
-  scene_info$lightintensity = light_prop_vec
-  scene_info$isflipped = flip_vec
-  scene_info$isvolume=fog_bool
-  scene_info$voldensity = fog_vec
-  scene_info$implicit_sample = implicit_vec
-  scene_info$order_rotation_list = order_rotation_list
   scene_info$clampval = clamp_value
-  scene_info$isgrouped = group_bool  
-  scene_info$group_transform= group_transform
-  scene_info$tri_normal_bools = tri_normal_bools
-  scene_info$is_tri_color = is_tri_color
-  scene_info$tri_color_vert= tri_color_vert
-  scene_info$fileinfo = objfilenamevec
-  scene_info$filebasedir = objbasedirvec
   scene_info$progress_bar = progress
   scene_info$numbercores = numbercores
   scene_info$hasbackground = hasbackground
   scene_info$background = backgroundstring
-  scene_info$scale_list = scale_factor
-  scene_info$sigmavec = sigmavec
   scene_info$rotate_env = rotate_env
   scene_info$intensity_env = intensity_env
   scene_info$verbose = verbose
   scene_info$debug_channel = debug_channel
-  scene_info$shared_id_mat=material_id
-  scene_info$is_shared_mat=material_id_bool
   scene_info$min_variance = min_variance
   scene_info$min_adaptive_size = min_adaptive_size
-  scene_info$glossyinfo = glossyinfo
-  scene_info$image_repeat = image_repeat
-  scene_info$csg_info = csg_info
-  scene_info$mesh_list=mesh_list
   scene_info$roughness_list = roughness_list
-  scene_info$animation_info = animation_info
-  
+
   all_info = list()
   all_info$scene_info = scene_info
   all_info$camera_info = camera_info
+  all_info$scene = scene
   
   return(all_info)
 }
