@@ -94,8 +94,10 @@ using namespace Rcpp;
 
 using namespace std;
 
+
+
 // [[Rcpp::export]]
-List render_scene_rcpp(List camera_info, List scene_info) {
+List render_scene_rcpp(List scene, List camera_info, List scene_info) {
 #ifdef DEBUG_MEMORY
   alloc = 0;
   // dealloc = 0;
@@ -105,63 +107,29 @@ List render_scene_rcpp(List camera_info, List scene_info) {
 
   //Unpack scene info
   bool ambient_light = as<bool>(scene_info["ambient_light"]);
-  IntegerVector type = as<IntegerVector>(scene_info["type"]);
-  NumericVector radius = as<NumericVector>(scene_info["radius"]);
   IntegerVector shape = as<IntegerVector>(scene_info["shape"]);
   List position_list = as<List>(scene_info["position_list"]);
-  List properties = as<List>(scene_info["properties"]);
-  int n = as<int>(scene_info["n"]);
   NumericVector bghigh  = as<NumericVector>(scene_info["bghigh"]);
   NumericVector bglow = as<NumericVector>(scene_info["bglow"]);
-  LogicalVector ischeckered = as<LogicalVector>(scene_info["ischeckered"]);
-  List checkercolors = as<List>(scene_info["checkercolors"]);
-  List gradient_info = as<List>(scene_info["gradient_info"]);
-  NumericVector noise = as<NumericVector>(scene_info["noise"]);
-  LogicalVector isnoise = as<LogicalVector>(scene_info["isnoise"]);
-  NumericVector noisephase = as<NumericVector>(scene_info["noisephase"]);
-  NumericVector noiseintensity = as<NumericVector>(scene_info["noiseintensity"]);
-  List noisecolorlist = as<List>(scene_info["noisecolorlist"]);
-  List angle = as<List>(scene_info["angle"]);
   LogicalVector isimage = as<LogicalVector>(scene_info["isimage"]);
   CharacterVector filelocation = as<CharacterVector>(scene_info["filelocation"]);
   List alphalist = as<List>(scene_info["alphalist"]);
-  NumericVector lightintensity = as<NumericVector>(scene_info["lightintensity"]);
-  LogicalVector isflipped = as<LogicalVector>(scene_info["isflipped"]);
-  LogicalVector isvolume = as<LogicalVector>(scene_info["isvolume"]);
-  NumericVector voldensity = as<NumericVector>(scene_info["voldensity"]);
-  LogicalVector implicit_sample = as<LogicalVector>(scene_info["implicit_sample"]);
-  List order_rotation_list = as<List>(scene_info["order_rotation_list"]);
   Float clampval = as<Float>(scene_info["clampval"]);
-  LogicalVector isgrouped = as<LogicalVector>(scene_info["isgrouped"]);
-  List group_transform = as<List>(scene_info["group_transform"]);
-  LogicalVector tri_normal_bools = as<LogicalVector>(scene_info["tri_normal_bools"]);
-  LogicalVector is_tri_color = as<LogicalVector>(scene_info["is_tri_color"]);
-  List tri_color_vert = as<List>(scene_info["tri_color_vert"]);
-  CharacterVector fileinfo = as<CharacterVector>(scene_info["fileinfo"]);
-  CharacterVector filebasedir = as<CharacterVector>(scene_info["filebasedir"]);
   bool progress_bar = as<bool>(scene_info["progress_bar"]);
   int numbercores = as<int>(scene_info["numbercores"]);
   bool hasbackground = as<bool>(scene_info["hasbackground"]);
   CharacterVector background = as<CharacterVector>(scene_info["background"]);
-  List scale_list = as<List>(scene_info["scale_list"]);
-  NumericVector sigmavec = as<NumericVector>(scene_info["sigmavec"]);
   Float rotate_env = as<Float>(scene_info["rotate_env"]);
   Float intensity_env = as<Float>(scene_info["intensity_env"]);
   bool verbose = as<bool>(scene_info["verbose"]);
   int debug_channel = as<int>(scene_info["debug_channel"]);
-  IntegerVector shared_id_mat = as<IntegerVector>(scene_info["shared_id_mat"]);
-  LogicalVector is_shared_mat = as<LogicalVector>(scene_info["is_shared_mat"]);
   Float min_variance = as<Float>(scene_info["min_variance"]);
   int min_adaptive_size = as<int>(scene_info["min_adaptive_size"]);
-  List glossyinfo = as<List>(scene_info["glossyinfo"]);
-  List image_repeat = as<List>(scene_info["image_repeat"]);
-  List csg_info = as<List>(scene_info["csg_info"]);
-  List mesh_list = as<List>(scene_info["mesh_list"]);
   List roughness_list = as<List>(scene_info["roughness_list"]);
-  List animation_info = as<List>(scene_info["animation_info"]);
 
   Environment pkg = Environment::namespace_env("rayrender");
   Function print_time = pkg["print_time"];
+  size_t n = shape.length();
   
 
   //Unpack Camera Info
@@ -215,7 +183,6 @@ List render_scene_rcpp(List camera_info, List scene_info) {
 
   CharacterVector bump_files = as<CharacterVector>(alphalist["bump_temp_file_names"]);
   LogicalVector has_bump = as<LogicalVector>(alphalist["bump_tex_bool"]);
-  NumericVector bump_intensity = as<NumericVector>(alphalist["bump_intensity"]);
 
   CharacterVector roughness_files = as<CharacterVector>(roughness_list["rough_temp_file_names"]);
   LogicalVector has_roughness = as<LogicalVector>(roughness_list["rough_tex_bool"]);
@@ -282,7 +249,7 @@ List render_scene_rcpp(List camera_info, List scene_info) {
 
   
   // size_t texture_bytes = 0;
-  for(int i = 0; i < n; i++) {
+  for(size_t i = 0; i < n; i++) {
     if(isimage(i)) {
       int nx, ny, nn;
       Float* tex_data = stbi_loadf(filelocation(i), &nx, &ny, &nn, 4);
@@ -324,7 +291,8 @@ List render_scene_rcpp(List camera_info, List scene_info) {
       nx_ny_nn_bump.push_back(nullptr);
     }
     if(has_roughness(i)) {
-      NumericVector temp_glossy = as<NumericVector>(glossyinfo(i));
+      List SingleMaterial = as<List>(scene(i))["material"];
+      NumericVector temp_glossy = as<NumericVector>(SingleMaterial["glossyinfo"]);
       int nxr, nyr, nnr;
       unsigned char * tex_data_roughness = stbi_load(roughness_files(i), &nxr, &nyr, &nnr, 0);
       // texture_bytes += nxr * nyr * nnr;
@@ -377,27 +345,15 @@ List render_scene_rcpp(List camera_info, List scene_info) {
   print_time(verbose, "Loaded Textures" );
   
   hitable_list imp_sample_objects;
-  std::shared_ptr<hitable> worldbvh = build_scene(type, radius, shape, position_list,
-                                properties,
-                                n,shutteropen,shutterclose,
-                                ischeckered, checkercolors,
-                                gradient_info,
-                                noise, isnoise, noisephase, noiseintensity, noisecolorlist,
-                                angle,
-                                isimage, has_alpha, alpha_textures, nx_ny_nn_alpha,
-                                textures, nx_ny_nn, has_bump, bump_textures, nx_ny_nn_bump,
-                                bump_intensity,
-                                roughness_textures, nx_ny_nn_roughness, has_roughness,
-                                lightintensity, isflipped,
-                                isvolume, voldensity, order_rotation_list,
-                                isgrouped, group_transform,
-                                tri_normal_bools, is_tri_color, tri_color_vert,
-                                fileinfo, filebasedir,
-                                scale_list, sigmavec, glossyinfo,
-                                shared_id_mat, is_shared_mat, shared_materials,
-                                image_repeat, csg_info, mesh_list, bvh_type, transformCache,
-                                animation_info, implicit_sample, imp_sample_objects,
-                                verbose, rng);
+  std::shared_ptr<hitable> worldbvh = build_scene(scene, shape, position_list,
+                                                  shutteropen,shutterclose,
+                                                  textures, nx_ny_nn,
+                                                  alpha_textures, nx_ny_nn_alpha,
+                                                  bump_textures, nx_ny_nn_bump,
+                                                  roughness_textures, nx_ny_nn_roughness,
+                                                  shared_materials, bvh_type,
+                                                  transformCache, imp_sample_objects,
+                                                  verbose, rng);
   print_time(verbose, "Built Scene BVH" );
 
   //Calculate world bounds and ensure camera is inside infinite area light
@@ -580,7 +536,6 @@ void PrintClassSizes() {
   Rcpp::Rcout << "---disk                  : " << sizeof(disk) << "\n";
   Rcpp::Rcout << "---cylinder              : " << sizeof(cylinder) << "\n";
   Rcpp::Rcout << "---ellipsoid             : " << sizeof(ellipsoid) << "\n";
-  Rcpp::Rcout << "---cone                  : " << sizeof(cone) << "\n";
   Rcpp::Rcout << "---curve                 : " << sizeof(curve) << "\n";
   Rcpp::Rcout << "---csg                   : " << sizeof(csg) << "\n";
   Rcpp::Rcout << "---plymesh               : " << sizeof(plymesh) << "\n";
